@@ -48,7 +48,7 @@ def init_plot(figsize):
 	return fig, axes
 
 
-def plot_snr_chi_pdf(coinc_param_distributions, instrument, binnedarray_string, snr_max, dynamic_range_factor = 1e-10, event_snr = None, event_chisq = None):
+def plot_snr_chi_pdf(coinc_param_distributions, instrument, binnedarray_string, snr_max, dynamic_range_factor = 1e-10, event_snr = None, event_chisq = None, sngls = None):
 	key = "%s_snr_chi" % instrument
 	if binnedarray_string == "LR":
 		binnedarray = getattr(coinc_param_distributions, "injection_pdf")[key]
@@ -56,6 +56,15 @@ def plot_snr_chi_pdf(coinc_param_distributions, instrument, binnedarray_string,
 		binnedarray = getattr(coinc_param_distributions, binnedarray_string)[key]
 	tag = {"background_pdf":"Noise", "injection_pdf":"Signal", "zero_lag_pdf":"Candidates", "LR":"LR"}[binnedarray_string]
 
+	# sngls is a sequence of {instrument: (snr, chisq)} dictionaries,
+	# obtain the co-ordinates for a sngls scatter plot for this
+	# instrument from that.  need to handle case in which there are no
+	# singles for this instrument
+	if sngls is not None:
+		sngls = numpy.array([sngl[instrument] for sngl in sngls if instrument in sngl])
+		if not len(sngls):
+			sngls = None
+
 	fig, axes = init_plot((8., 8. / plotutil.golden_ratio))
 	# the last bin can have a centre at infinity, and its value is
 	# always 0 anyway so there's no point in trying to include it
@@ -67,7 +76,8 @@ def plot_snr_chi_pdf(coinc_param_distributions, instrument, binnedarray_string,
 		denom_binnedarray = coinc_param_distributions.background_pdf[key]
 		assert (denom_binnedarray.bins[0].centres()[:-1] == x).all()
 		assert (denom_binnedarray.bins[1].centres()[:-1] == y).all()
-		z /= denom_binnedarray.array[:-1,:-1]
+		# note:  don't do this in-place
+		z = z / denom_binnedarray.array[:-1,:-1]
 	if numpy.isnan(z).any():
 		warnings.warn("%s PDF contains NaNs" % instrument)
 		z = numpy.ma.masked_where(numpy.isnan(z), z)
@@ -83,27 +93,36 @@ def plot_snr_chi_pdf(coinc_param_distributions, instrument, binnedarray_string,
 	y = y[binnedarray.bins[xlo:xhi, ylo:yhi][1]]
 	z = z[binnedarray.bins[xlo:xhi, ylo:yhi]]
 
-	if binnedarray_string == "LR":
-		# because it's usually more convenient to display the
-		# natural log of the likelihood ratio, it's helpful to plot
-		# that here.  if we relied on a logarithmic colour bar we
-		# end up with power-of-10 contours instead of power-of-e
-		# contours, so we can't do that.
+	# because it's usually more convenient to display the natural log
+	# of the likelihood ratio, it's helpful to plot that here.  if we
+	# relied on a logarithmic colour bar we end up with power-of-10
+	# contours instead of power-of-e contours, so we can't do that.
+	with numpy.errstate(divide = "ignore"):
 		z = numpy.log(z)
-		norm = matplotlib.colors.Normalize()
-	else:
-		# matplotlib's colour bar seems to rely on being able to
-		# store the ratio of the lowest and highest value in a
-		# double so the range cannot be more than about 300 orders
-		# of magnitude.  experiments show it starts to go wrong
-		# before that but 250 orders of magnitude seems to be OK
-		numpy.clip(z, z.max() * dynamic_range_factor, float("+inf"), out = z)
-		norm = matplotlib.colors.LogNorm()
+
+	if binnedarray_string == "LR":
+		norm = matplotlib.colors.Normalize(vmin = -80., vmax = +200.)
+		levels = numpy.linspace(-80., +200, 141)
+	elif binnedarray_string == "background_pdf":
+		norm = matplotlib.colors.Normalize(vmin = -30., vmax = z.max())
+		levels = 50
+	elif binnedarray_string == "injection_pdf":
+		norm = matplotlib.colors.Normalize(vmin = -60., vmax = z.max())
+		levels = 50
+	else:	# binnedarray_string == "zero_lag_pdf":
+		norm = matplotlib.colors.Normalize(vmin = -30., vmax = z.max())
+		levels = 50
 
 	mesh = axes.pcolormesh(x, y, z.T, norm = norm, cmap = "afmhot", shading = "gouraud")
-	axes.contour(x, y, z.T, norm = norm, colors = "k", linewidths = .5)
+	if binnedarray_string == "LR":
+		cs = axes.contour(x, y, z.T, levels, norm = norm, colors = "k", linewidths = .5, alpha = .3)
+		axes.clabel(cs, [-20., -10., 0., +10., +20.], fmt = "%g", fontsize = 8)
+	else:
+		axes.contour(x, y, z.T, levels, norm = norm, colors = "k", linestyles = "-", linewidths = .5, alpha = .3)
 	if event_snr is not None and event_chisq is not None:
 		axes.plot(event_snr, event_chisq / event_snr / event_snr, 'ko', mfc = 'None', mec = 'g', ms = 14, mew=4)
+	if sngls is not None:
+		axes.plot(sngls[:,0], sngls[:,1] / sngls[:,0]**2., "b.", alpha = .2)
 	axes.loglog()
 	axes.grid(which = "both")
 	#axes.set_xlim((xlo, xhi))
@@ -112,11 +131,11 @@ def plot_snr_chi_pdf(coinc_param_distributions, instrument, binnedarray_string,
 	axes.set_xlabel(r"$\mathrm{SNR}$")
 	axes.set_ylabel(r"$\chi^{2} / \mathrm{SNR}^{2}$")
 	if tag.lower() in ("signal",):
-		axes.set_title(r"%s %s $P(\chi^{2} / \mathrm{SNR}^{2} | \mathrm{SNR})$" % (instrument, tag))
+		axes.set_title(r"$\ln P(\chi^{2} / \mathrm{SNR}^{2} | \mathrm{SNR}, \mathrm{%s})$ in %s" % (tag.lower(), instrument))
 	elif tag.lower() in ("noise", "candidates"):
-		axes.set_title(r"%s %s $P(\mathrm{SNR}, \chi^{2} / \mathrm{SNR}^{2})$" % (instrument, tag))
+		axes.set_title(r"$\ln P(\mathrm{SNR}, \chi^{2} / \mathrm{SNR}^{2} | \mathrm{%s})$ in %s" % (tag.lower(), instrument))
 	elif tag.lower() in ("lr",):
-		axes.set_title(r"%s $\ln P(\chi^{2} / \mathrm{SNR}^{2} | \mathrm{SNR}, \mathrm{signal} ) / P(\mathrm{SNR}, \chi^{2} / \mathrm{SNR}^{2} | \mathrm{noise} )$" % instrument)
+		axes.set_title(r"$\ln P(\chi^{2} / \mathrm{SNR}^{2} | \mathrm{SNR}, \mathrm{signal} ) / P(\mathrm{SNR}, \chi^{2} / \mathrm{SNR}^{2} | \mathrm{noise})$ in %s" % instrument)
 	else:
 		raise ValueError(tag)
 	return fig
@@ -193,11 +212,12 @@ def plot_rates(coinc_param_distributions, ranking_data = None):
 	except AttributeError:
 		return fig
 
-def plot_snr_joint_pdf(coinc_param_distributions, instruments, horizon_distances, max_snr, ifo_snr = {}):
+
+def plot_snr_joint_pdf(coinc_param_distributions, instruments, horizon_distances, max_snr, ifo_snr = {}, sngls = None):
 	if len(instruments) > 2:
 		# FIXME:  figure out how to plot 3D PDFs
 		return None
-	ignored, binnedarray, ignored = coinc_param_distributions.snr_joint_pdf_cache[(instruments, horizon_distances)]
+	ignored, binnedarray, ignored = coinc_param_distributions.snr_joint_pdf_cache[coinc_param_distributions.snr_joint_pdf_keyfunc(instruments, horizon_distances)]
 	instruments = sorted(instruments)
 	horizon_distances = dict(horizon_distances)
 	fig, axes = init_plot((5, 4))
@@ -208,6 +228,15 @@ def plot_snr_joint_pdf(coinc_param_distributions, instruments, horizon_distances
 		warnings.warn("%s SNR PDF for %s contains NaNs" % (", ".join(instruments), ", ".join("%s=%g" % instdist for instdist in sorted(horizon_distances.items()))))
 		z = numpy.ma.masked_where(numpy.isnan(z), z)
 
+	# sngls is a sequence of {instrument: (snr, chisq)} dictionaries,
+	# obtain the co-ordinates for a sngls scatter plot for the
+	# instruments from that.  need to handle case in which there are no
+	# singles for this instrument
+	if sngls is not None:
+		sngls = numpy.array([(sngl[instruments[0]][0], sngl[instruments[1]][0]) for sngl in sngls if sorted(sngl) == instruments])
+		if not len(sngls):
+			sngls = None
+
 	# the range of the plots
 	xlo, xhi = far.ThincaCoincParamsDistributions.snr_min, max_snr
 
@@ -215,23 +244,30 @@ def plot_snr_joint_pdf(coinc_param_distributions, instruments, horizon_distances
 	y = y[binnedarray.bins[xlo:xhi, xlo:xhi][1]]
 	z = z[binnedarray.bins[xlo:xhi, xlo:xhi]]
 
-	# don't try to plot blank PDFs (it upsets older matplotlibs)
-	if z.max() == 0.:
-		return None
-
-	# these plots only require about 20 orders of magnitude of dynamic
-	# range
-	numpy.clip(z, z.max() * 1e-20, float("+inf"), out = z)
-
 	# one last check for craziness to make error messages more
 	# meaningful
 	assert not numpy.isnan(z).any()
-	assert not (z <= 0.).any()
+	assert not (z < 0.).any()
 
-	mesh = axes.pcolormesh(x, y, z.T, norm = matplotlib.colors.LogNorm(), cmap = "afmhot", shading = "gouraud")
-	axes.contour(x, y, z.T, norm = matplotlib.colors.LogNorm(), colors = "k", linewidths = .5)
+	# plot the natural logarithm of the PDF
+	with numpy.errstate(divide = "ignore"):
+		z = numpy.log(z)
+
+	# FIXME:  hack to allow all-0 PDFs to be plotted.  remove when we
+	# have a version of matplotlib that doesn't crash, whatever version
+	# of matplotlib that is
+	if numpy.isinf(z).all():
+		z[:,:] = -60.
+		z[0,0] = -55.
+
+	norm = matplotlib.colors.Normalize(vmin = -40., vmax = max(0., z.max()))
+
+	mesh = axes.pcolormesh(x, y, z.T, norm = norm, cmap = "afmhot", shading = "gouraud")
+	axes.contour(x, y, z.T, 50, norm = norm, colors = "k", linestyles = "-", linewidths = .5, alpha = .3)
 	if ifo_snr:
 		axes.plot(ifo_snr[instruments[0]], ifo_snr[instruments[1]], 'ko', mfc = 'None', mec = 'g', ms = 14, mew=4)
+	if sngls is not None:
+		axes.plot(sngls[:,0], sngls[:,1], "b.", alpha = .2)
 	axes.loglog()
 	axes.grid(which = "both", linestyle = "-", linewidth = 0.2)
 	#axes.set_xlim((xlo, xhi))
@@ -240,7 +276,7 @@ def plot_snr_joint_pdf(coinc_param_distributions, instruments, horizon_distances
 	# co-ordinates are in alphabetical order
 	axes.set_xlabel(r"$\mathrm{SNR}_{\mathrm{%s}}$" % instruments[0])
 	axes.set_ylabel(r"$\mathrm{SNR}_{\mathrm{%s}}$" % instruments[1])
-	axes.set_title(r"$P(%s)$" % ", ".join("\mathrm{SNR}_{\mathrm{%s}}" % instrument for instrument in instruments))
+	axes.set_title(r"$\ln P(%s)$" % ", ".join("\mathrm{SNR}_{\mathrm{%s}}" % instrument for instrument in instruments))
 
 	return fig
 	
@@ -267,7 +303,6 @@ def plot_likelihood_ratio_pdf(ranking_data, instruments, (xlo, xhi), tag, binned
 	ylo = pdf[xlo:xhi,].min()
 	if zerolag_pdf is not None:
 		yhi = max(yhi, zerolag_pdf[xlo:xhi,].max())
-		ylo = min(ylo, zerolag_pdf[xlo:xhi,].min())
 	ylo = max(yhi * 1e-40, ylo)
 	axes.set_ylim((10**math.floor(math.log10(ylo) - .5), 10**math.ceil(math.log10(yhi) + .5)))
 	axes.set_xlim((xlo, xhi))
@@ -278,7 +313,7 @@ def plot_likelihood_ratio_pdf(ranking_data, instruments, (xlo, xhi), tag, binned
 	except AttributeError:
 		return fig
 
-def plot_likelihood_ratio_ccdf(fapfar, (xlo, xhi), zerolag_ln_likelihood_ratios = None, event_likelihood = None):
+def plot_likelihood_ratio_ccdf(fapfar, (xlo, xhi), zerolag_ln_likelihood_ratios = None, event_ln_likelihood_ratio = None):
 	fig, axes = init_plot((8., 8. / plotutil.golden_ratio))
 
 	ccdf = fapfar.ccdf_interpolator
@@ -296,8 +331,8 @@ def plot_likelihood_ratio_ccdf(fapfar, (xlo, xhi), zerolag_ln_likelihood_ratios
 		y = zerolag_ln_likelihood_ratios[:,1]
 		axes.semilogy(x, y, color = "k", linestyle = "", marker = "+")
 
-	if event_likelihood is not None:
-		axes.axvline(event_likelihood, ylo, yhi)
+	if event_ln_likelihood_ratio is not None:
+		axes.axvline(event_ln_likelihood_ratio, ylo, yhi)
 
 	axes.set_xlim((xlo, xhi))
 	axes.set_ylim((ylo, yhi))
@@ -313,6 +348,8 @@ def plot_likelihood_ratio_ccdf(fapfar, (xlo, xhi), zerolag_ln_likelihood_ratios
 		return fig
 
 def plot_horizon_distance_vs_time(coinc_param_distributions, (tlo,thi), tbins, colours = {"H1": "r", "H2": "b", "L1": "g", "V1": "m"}):
+	tlo, thi = float(tlo), float(thi)
+
 	horizon_history = coinc_param_distributions.horizon_history
 
 	fig, axes = init_plot((8., 8. / plotutil.golden_ratio))
@@ -324,9 +361,9 @@ def plot_horizon_distance_vs_time(coinc_param_distributions, (tlo,thi), tbins, c
 		yhi = max(y.max()+5., yhi)
 	axes.set_ylim((0,yhi))
 	axes.set_xlim((round(tlo), round(thi)))
-	axes.set_ylabel('D_H (Mpc)')
+	axes.set_ylabel('Horizon Distance (Mpc)')
 	axes.set_xlabel('GPS Time (s)')
-	axes.set_title('Horizon Distance')
+	axes.set_title('Horizon Distance vs.\ Time')
 	axes.legend(loc = "lower left")
 	#FIXME: remove when we have a new enough matplotlib on all the reference platforms
 	try:	
